/*
 * 3-D Helixpaint. Inspired by Electropaint.
 */

#include <stdlib.h>
#include <math.h>

#include <gtk/gtk.h>

#include <gtk/gtkgl.h>

#include "midi-source.h"

typedef struct _HelixSlotParams  HelixSlotParams;
typedef struct _HelixIndexZ      HelixIndexZ;
typedef struct _HelixAttribs     HelixAttribs;
typedef struct _HelixElement     HelixElement;
typedef struct _HelixWindow      HelixWindow;

#define HELIX_TYPE_SLOT_TYPE (helix_slot_type_get_type ())

GType helix_slot_type_get_type (void) G_GNUC_CONST;

typedef enum
{
  HELIX_ARMLEN_ABS,
  HELIX_ARMLEN_REL,
  HELIX_ARMLEN_AMP,
  HELIX_HEIGHT_ABS,
  HELIX_HEIGHT_REL,
  HELIX_HEIGHT_AMP,
  HELIX_SIZE_ABS,
  HELIX_SIZE_REL,
  HELIX_SIZE_AMP,
  HELIX_ANGLE_ABS,
  HELIX_ANGLE_REL,
  HELIX_ANGLE_AMP,
  HELIX_ROLL_ABS,
  HELIX_ROLL_REL,
  HELIX_ROLL_AMP,
  HELIX_WAVE_ABS,
  HELIX_WAVE_REL,
  HELIX_WAVE_AMP,
  HELIX_WAG_ABS,
  HELIX_WAG_REL,
  HELIX_WAG_AMP,

  HELIX_L_ABS,
  HELIX_L_REL,
  HELIX_L_AMP,
  HELIX_A_ABS,
  HELIX_A_REL,
  HELIX_A_AMP,
  HELIX_B_ABS,
  HELIX_B_REL,
  HELIX_B_AMP,
  HELIX_OPACITY_ABS,
  HELIX_OPACITY_REL,
  HELIX_OPACITY_AMP,

  HELIX_N_VERTICES,

  HELIX_VIEW_ROT_X,
  HELIX_VIEW_ROT_Y,
  HELIX_VIEW_ROT_Z,
  HELIX_ROTATION_REL,
  HELIX_FADEOUT,

  HELIX_SLOT_LAST    /*< skip >*/
} HelixSlotType;


struct _HelixSlotParams
{
  HelixSlotType  slot;
  gdouble        lower;
  gdouble        upper;
  gdouble        value;
  gint           midi_ctrl;
  gboolean       cyclic;
  gchar         *name;
};


struct _HelixIndexZ
{
  gdouble z_depth;
  gint    index;
};


struct _HelixAttribs
{
  gdouble  armlen;
  gdouble  height;
  gdouble  size;
  gdouble  angle;
  gdouble  roll;
  gdouble  wave;
  gdouble  wag;
  gdouble  opacity;

  gdouble  Lab[3];

  gdouble  vertices;
};


struct _HelixElement
{
  gdouble      z_depth;  /* for proper alpha blending */
  gdouble      transform[16];

  /* state for drawing */
  HelixAttribs current;

  /* state at next tick */
  HelixAttribs absolute;
  HelixAttribs relative;
};


struct _HelixWindow
{
  GtkWidget     *window;
  GtkWidget     *drawing_area;
  GTimer        *timer;
  guint          timeout_id;
  gboolean       visibility;

  GtkWidget     *control_panel;

  MidiSource    *midi_source;

  gint           frames;
  gdouble        fps;
  gdouble        ticks;
  gdouble        tick_fraction;

  HelixElement  *master_element;
  HelixAttribs   oscillation_params;

  GdkPixbuf     *texture;
  gboolean       textured;

  gdouble        rotation_z;

  gint           nelements;
  HelixElement  *elements;
  GtkAdjustment *adjustments[HELIX_SLOT_LAST];
};


GtkWidget * helix_create_control_panel (HelixWindow *hwin);

